<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
                xmlns:l="http://www.landxml.org/schema/LandXML-1.2"
                xmlns:h="http://xml.hexagon.com/schema/HeXML-1.7"
                xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                exclude-result-prefixes="h l">
  <!-- Source template ID: {E7DC6404-29A2-4147-8AFF-03B84622F666} -->
  <!-- Source version: 0.1 -->
  <!-- This stylesheet export the point codes in the code attribute of the CgPoint
       with attributes in name:value format --> 
  
  <!-- Output HTML doctype with text/html content-type and without XML declaration-->
  <xsl:output method="xml" omit-xml-declaration="no" encoding="utf-8" />

  <!-- Constants variables read by Viva exporter-->
  <xsl:variable name="fileExt" select="'xml'"/>
  <xsl:variable name="fileDesc" select="'Creates a LandXML file suitable for import into AutoCad Civil3D'" />

  <!-- Key to search a point code from CgPoint data -->
  <xsl:key name="KeyPointCode" match="h:Point/h:PointCode" use="concat(../@uniqueID,'+',@code)"/>
  <xsl:key name="Backsight" match="l:Backsight" use="@setupID"/>

  <!-- This is an identity template - it copies everything that doesn't match another template -->
  <xsl:template match="@* | node()">
    <xsl:copy>
      <xsl:apply-templates select="@* | node()" />
    </xsl:copy>
  </xsl:template>

  <!-- Do not write the hexagon extension -->
  <xsl:template match="h:HexagonLandXML"/>

  <!-- Replace CgPoint code -->
  <xsl:template match="l:CgPoint">
    <xsl:copy>
      <xsl:variable name ="point" select="key('KeyPointCode', concat(@name,'+',@code))"/>
      <xsl:if test="$point">
        <xsl:attribute name="code">
          <xsl:apply-templates select="$point"/>
        </xsl:attribute>
        <xsl:apply-templates select="@*[not(name()='code')] | node()" />
      </xsl:if>
      <xsl:if test="not($point)">
        <xsl:apply-templates select="@* | node()" />
      </xsl:if>
    </xsl:copy>
  </xsl:template>
  
  <!-- ObservationGroup add setupID -->
  <xsl:template match="l:ObservationGroup">
  <xsl:variable name="firstRawObs" select="l:RawObservation[1]"/>
    <xsl:copy>
        <xsl:attribute name="setupID">
          <xsl:value-of select="$firstRawObs/@setupID"/>
        </xsl:attribute>
        <xsl:apply-templates select="@* | node()" />
      </xsl:copy>
  </xsl:template>
  
  <!-- write BS for alpha keys -->
  <xsl:template match="l:InstrumentSetup">
    <xsl:param name="InstrumentSetup"/>
    <xsl:copy>
      <xsl:apply-templates select="@* | node()" />    
      <xsl:variable name ="backsight" select="key('Backsight', @id)"/>
      <xsl:if test="$backsight">
        <xsl:apply-templates select="$backsight">
          <xsl:with-param name="InstrumentSetup" select="1"/>
        </xsl:apply-templates>
      </xsl:if>
    </xsl:copy>
  </xsl:template>
  
  <!-- BS alpha numeric keys -->
  <xsl:template match="l:Backsight">
    <xsl:param name="InstrumentSetup"/>
    <xsl:choose>      
      <xsl:when test="not($InstrumentSetup)">
        <xsl:if test="number(l:BacksightPoint/@name) =l:BacksightPoint/@name">
          <xsl:copy>
            <xsl:apply-templates select="@* | node()" />     
          </xsl:copy>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
       <xsl:if test="not(number(l:BacksightPoint/@name) =l:BacksightPoint/@name)">
          <xsl:copy>
            <xsl:apply-templates select="@* | node()" />     
          </xsl:copy>
       </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- Match free and thematical codes with attribute 'code'.
       (attribute 'code' do not exist in the elements of the Codelist -->
  <xsl:template match="h:FreeCode[@code] | h:PointCode[@code] | h:LineCode[@code] | h:AreaCode[@code]">
    <xsl:variable name="CodeInfo" select="@codeInformation"/>
	<xsl:if test="$CodeInfo">
	  <xsl:value-of select="concat(@code, ' ', @codeInformation)"/>
	</xsl:if>
	<xsl:if test="not($CodeInfo)">
	  <xsl:value-of select="@code"/>
	</xsl:if>
    <xsl:apply-templates select="h:Attribute"/>
    <xsl:apply-templates select="../h:Annotations"/>
  </xsl:template>

  <!-- Write attribute as attrName:attrValue -->
  <xsl:template match="h:Attribute">
    <xsl:value-of select="concat(' ', @name, ' ', @value)"/>
  </xsl:template>

<!-- Template for any Annotation -->  
  <xsl:template match="h:Annotations">    
    <xsl:variable name="Annotation1" select="@annotation1"/>
	<xsl:if test="$Annotation1">
	  <xsl:value-of select="concat(' ', @annotation1)"/>
	</xsl:if> 
	<xsl:variable name="Annotation2" select="@annotation2"/>
	<xsl:if test="$Annotation2">
	  <xsl:value-of select="concat(' ', @annotation2)"/>
	</xsl:if> 
	<xsl:variable name="Annotation3" select="@annotation3"/>
	<xsl:if test="$Annotation3">
	  <xsl:value-of select="concat(' ', @annotation3)"/>
	</xsl:if> 
	<xsl:variable name="Annotation4" select="@annotation4"/>
	<xsl:if test="$Annotation4">
	  <xsl:value-of select="concat(' ', @annotation4)"/>
	</xsl:if>        
  </xsl:template>

</xsl:stylesheet>