<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
                xmlns:l="http://www.landxml.org/schema/LandXML-1.2"
                xmlns:h="http://xml.hexagon.com/schema/HeXML-1.7"
                xmlns:r="urn:swdc:resources"
                xmlns="http://www.landxml.org/schema/LandXML-1.2"
                exclude-result-prefixes="h l r">

  <!-- Output HTML doctype with text/html content-type and without XML declaration-->
  <xsl:output method="xml" omit-xml-declaration="no" encoding="utf-8"/>

  <!-- Constants variables read by Viva exporter-->
  <xsl:variable name="fileExt" select="'xml'"/>
  <xsl:variable name="fileDesc" select="'Creates a LandXML file suitable for import into Bentley InRoads'" />

  <!-- This is an identity template - it copies everything that doesn't match another template -->
  <xsl:template match="@* | node()">
    <xsl:copy>
      <xsl:apply-templates select="@* | node()" />
    </xsl:copy>
  </xsl:template>

  <!-- Create a collection of points used in a curve and index by its pntRef-->
  <xsl:key name="KeyCurvePoint" match="h:Curve/h:Start | h:Curve/l:Start" use="@pntRef"/>
  <xsl:key name="KeyCurvePoint" match="h:Curve/h:Mid" use="@pntRef"/>
  <xsl:key name="KeyCurvePoint" match="h:Curve/h:End | h:Curve/l:End" use="@pntRef"/>
  
  <!-- Key to search a point code from CgPoint data -->
  <xsl:key name="KeyPointCode" match="h:Point/h:PointCode" use="concat(../@uniqueID,'+',@code)"/>
  
  <!-- Key to search CgPoint data -->
  <xsl:key name="KeyCgPoint" match="l:CgPoint" use="@name"/>
  
  <!-- Key to search points for Chain in coord geom data -->
  <xsl:key name="KeyCurve" match="h:Curve" use="@oID"/>
  <xsl:key name="KeyBestFitCurve" match="h:BestFitCurve" use="@oID"/>
  <xsl:key name="KeySpline" match="h:Spline" use="@oID"/>

  <!-- Template for CgPoints used in a curve-->
  <xsl:template match="l:CgPoint[key('KeyCurvePoint',@name)]">
    <xsl:param name="Code"/>
    <xsl:copy>
      <xsl:attribute name="pointGeometry">curve</xsl:attribute>
      <xsl:variable name ="point" select="key('KeyPointCode', concat(@name,'+',@code))"/>
      <xsl:if test="$point">
        <xsl:attribute name="code">
          <xsl:apply-templates select="$point">
            <xsl:with-param name="Code" select="1"/>
          </xsl:apply-templates>
        </xsl:attribute>
        <xsl:apply-templates select="@*[not(name()='code')] | node()" />
      </xsl:if>
      <xsl:if test="not($point)">
        <xsl:apply-templates select="@* | node()" />
      </xsl:if>
    </xsl:copy>
  </xsl:template>

  <!-- Template for any CgPoint -->
  <xsl:template match="l:CgPoint">
    <xsl:param name="Code"/>
    <xsl:param name="inSearch"/>
    <xsl:copy>
      <xsl:variable name="cgPointName" select="@name"/>
      <xsl:variable name="inChain">
        <!-- Looks for a Chain containing the point name -->
        <xsl:apply-templates select="//h:Chain[contains(., $cgPointName)] | //l:Chain[contains(., $cgPointName)]">
          <xsl:with-param name="pointName" select="$cgPointName"/>
          <xsl:with-param name="inSearch" select="1"/>
        </xsl:apply-templates>
      </xsl:variable>
      <xsl:choose>
        <xsl:when test="starts-with($inChain,'isFound')">
          <xsl:attribute name="pointGeometry">curve</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:attribute name="pointGeometry">point</xsl:attribute>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:variable name ="point" select="key('KeyPointCode', concat(@name,'+',@code))"/>
      <xsl:if test="$point">
        <xsl:attribute name="code">
          <xsl:apply-templates select="$point">
            <xsl:with-param name="Code" select="1"/>
          </xsl:apply-templates>
        </xsl:attribute>
        <xsl:apply-templates select="@*[not(name()='code')] | node()" />
      </xsl:if>
      <xsl:if test="not($point)">
        <xsl:apply-templates select="@* | node()" />
      </xsl:if>
    </xsl:copy>
  </xsl:template>
  
  <!-- Template for any CoordGeom -->
  <xsl:template match="l:CoordGeom" name="CoordGeom">
    <xsl:param name="Chain"/>
    <xsl:param name="Last"/>
    <xsl:choose>      
      <xsl:when test="not($Chain)">
        <CoordGeom>
      	  <!--<xsl:apply-templates select="@* | *"/>-->
      	  <Chain>
      	    <xsl:call-template name="CoordGeom"><xsl:with-param name="Chain" select="1"/></xsl:call-template>
      	    <xsl:apply-templates select="child::*[last()]/l:End[last()]">
      	      <xsl:with-param name="Chain" select="1"/>
      	      <xsl:with-param name="Last" select="1"/>
      	    </xsl:apply-templates>
      	  </Chain>
		</CoordGeom>
  	  </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="@* | node()"><xsl:with-param name="Chain" select="1"/></xsl:apply-templates>
      </xsl:otherwise>
	</xsl:choose>
  </xsl:template>
  
    <!-- Template for any Parcel  convert them as Planfeature -->
  <xsl:template match="l:Parcels">
    <PlanFeatures>
      <xsl:apply-templates select="@* | node()"/>
    </PlanFeatures>
  </xsl:template>
  
  <!-- Template for any Parcel  convert them as Planfeature -->
  <xsl:template match="l:Parcel">
    <PlanFeature>
      <xsl:apply-templates select="@*[name()='name'] | node()"/>
    </PlanFeature>
  </xsl:template>
    
  <!-- Template for any l:Line -->  
  <xsl:template match="l:Line">
    <xsl:param name="Chain"/>
    <xsl:choose>      
      <xsl:when test="not($Chain)">
        <xsl:copy>
          <xsl:apply-templates select="@* | node()" />
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="@* [name()='pntRef' ]| node()" ><xsl:with-param name="Chain" select="1"/></xsl:apply-templates>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!-- Template for any l:Curve-->
  <xsl:template match="l:Curve">
    <xsl:param name="Chain"/>
    <xsl:choose>      
      <xsl:when test="not($Chain)">
        <xsl:copy>
          <xsl:apply-templates select="@* | node()" />
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>		
		<xsl:variable name ="curve" select="key('KeyCurve', @oID)"/>
		<xsl:variable name ="bestfitcurve" select="key('KeyBestFitCurve', @oID)"/>
		<xsl:if test="$curve">
		  <!--normal arc - get start,  mid and end form h:Curve-->
		  <xsl:apply-templates select="$curve"><xsl:with-param name="Chain" select="1"/></xsl:apply-templates>
		</xsl:if>
		<xsl:if test="$bestfitcurve">
		  <!--bestfit arc - get start, end and chain form h:Curve-->
		  <xsl:apply-templates select="@* [name()='pntRef' ]| node()" ><xsl:with-param name="Chain" select="1"/></xsl:apply-templates>
		  <xsl:apply-templates select="$bestfitcurve"><xsl:with-param name="Chain" select="1"/></xsl:apply-templates>
		</xsl:if>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
    
  <!-- Template for l:IregularLine-->
  <xsl:template match="l:IrregularLine">
    <xsl:param name="Chain"/>
    <xsl:choose>      
      <xsl:when test="not($Chain)">
        <xsl:copy>
          <xsl:apply-templates select="@* | node()" />
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>
		 <xsl:variable name ="spline" select="key('KeySpline', @oID)"/>
        <xsl:apply-templates select="$spline">
          <xsl:with-param name="Chain" select="1"/>          
        </xsl:apply-templates>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!-- Template for l:Start-->
  <xsl:template match="l:Start">
    <xsl:param name="Chain"/>
    <xsl:choose>      
      <xsl:when test="not($Chain)">
              <xsl:copy>              
                <xsl:apply-templates select="@* | node()" />
              </xsl:copy>
      </xsl:when>
    <xsl:otherwise>
        <xsl:value-of select="concat(@pntRef, ' ')"/>
      </xsl:otherwise>
    </xsl:choose>    
  </xsl:template>
  
    <!-- Template for l:Center-->
  <xsl:template match="l:Center">
    <xsl:param name="Chain"/>          
      <xsl:if test="not($Chain)">
              <xsl:copy>              
                <xsl:apply-templates select="@* | node()" />
              </xsl:copy>
      </xsl:if>    
  </xsl:template>
  
  <!-- Template for l:End-->
  <xsl:template match="l:End">
    <xsl:param name="Chain"/>
    <xsl:param name="Last"/>
    <xsl:choose>      
      <xsl:when test="not($Chain)">
              <xsl:copy>              
                <xsl:apply-templates select="@* | node()" />
              </xsl:copy>
      </xsl:when>
    <xsl:otherwise>
        <xsl:if test="$Last">
        <xsl:value-of select="@pntRef"/>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>    
  </xsl:template>  
  
<!-- Template for l:ObsercationGroup with missing Backsight workaround-->
  <xsl:template match="l:ObservationGroup[not(l:Backsight)]">
    <ObservationGroup>
      <xsl:attribute name="id">
        <xsl:value-of select="@id"/>
      </xsl:attribute>
      <xsl:apply-templates select="l:TargetPoint"/>
      <xsl:variable name="firstRawObs" select="l:RawObservation[1]"/>
      <Backsight>
        <xsl:attribute name="azimuth">
          <xsl:value-of select="$firstRawObs/@horizAngle"/>
        </xsl:attribute>
        <xsl:attribute name="targetHeight">
          <xsl:value-of select="$firstRawObs/@targetHeight"/>
        </xsl:attribute>
        <xsl:attribute name="circle">
          <xsl:value-of select="$firstRawObs/@horizAngle"/>
        </xsl:attribute>
        <xsl:attribute name="setupID">
          <xsl:value-of select="$firstRawObs/@setupID"/>
        </xsl:attribute>
        <BacksightPoint>
            <xsl:attribute name="name">
              <xsl:value-of select="$firstRawObs/l:TargetPoint/@name"/>
            </xsl:attribute>
            <xsl:attribute name="pntRef">
              <xsl:value-of select="$firstRawObs/l:TargetPoint/@pntRef"/>
            </xsl:attribute>
            <xsl:value-of select="$firstRawObs/l:TargetPoint/text()"/>
          </BacksightPoint>
      </Backsight>
      <xsl:apply-templates select="l:RawObservation"/>
    </ObservationGroup>
  </xsl:template>
  
 <!-- Templates for HeXML part-->  
  
  <!-- Template for any h:Parcel  convert them as Planfeature -->
  <xsl:template match="h:Parcel">
    <xsl:param name="Parcel"/>
    <xsl:element name="PlanFeature" namespace="http://xml.hexagon.com/schema/HeXML-1.7">
      <xsl:attribute name="name">@name</xsl:attribute>
      <xsl:variable name="Color" select="@areaBorderColor"/>
      <xsl:if test="$Color">
	    <xsl:attribute name="lineColor">
	      <xsl:value-of select="$Color"/>
	    </xsl:attribute>
	  </xsl:if>
	  <xsl:variable name="Style" select="@areaBorderStyle"/>
      <xsl:if test="$Style">
	    <xsl:attribute name="lineStyle">
	      <xsl:value-of select="$Style"/>
	    </xsl:attribute>
	  </xsl:if>
	  <xsl:variable name="Width" select="@areaBorderWidth"/>
	  <xsl:if test="$Width">
	    <xsl:attribute name="lineWidth">
	      <xsl:value-of select="$Width"/>
	    </xsl:attribute>
	  </xsl:if>
	  <xsl:apply-templates select="@* [not(name()='areaBorderColor') and not(name()='areaBorderStyle') and not(name()='areaBorderStyle') and not(name()='areaBorderWidth') and not(name()='areaShadingStyle') and not(name()='areaShadingColor')] | *">
        <xsl:with-param name="Parcel" select="1"/>
      </xsl:apply-templates>
    </xsl:element>
  </xsl:template>
  
  <!-- Template for any h:AreaCode  convert them as LineCode
       (attribute 'code' do not exist in the elements of the Codelist -->
  <xsl:template match="h:AreaCode[@code]">
    <xsl:param name="Parcel"/>
    <xsl:choose>
		<xsl:when test="not($Parcel)">
          <xsl:copy>
            <xsl:apply-templates select="@* | node()" />
          </xsl:copy>
        </xsl:when>
        <xsl:otherwise>
          <xsl:element name="LineCode" namespace="http://xml.hexagon.com/schema/HeXML-1.7">
            <xsl:attribute name="name">@name</xsl:attribute>
            <xsl:apply-templates select="@* | *"/>
          </xsl:element>
        </xsl:otherwise>
	</xsl:choose>
  </xsl:template>
  
  <!-- Template for h:Curve-->
  <xsl:template match="h:Curve">
    <xsl:param name="Chain"/>
    <xsl:choose>      
      <xsl:when test="not($Chain)">
        <xsl:copy>
          <xsl:apply-templates select="@* | node()" />
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="@* [name()='pntRef'] | node()" >
          <xsl:with-param name="Chain" select="1"/>
        </xsl:apply-templates>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!-- Template for h.BestFitCurve-->
  <xsl:template match="h:BestFitCurve">
    <xsl:param name="Chain"/>
    <xsl:choose>      
      <xsl:when test="not($Chain)">
        <xsl:copy>
          <xsl:apply-templates select="@* | node()" />
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="@* [name()='Chain'] | node()" >
          <xsl:with-param name="Chain" select="1"/>
        </xsl:apply-templates>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!-- Template for h.Spline-->
  <xsl:template match="h:Spline">
    <xsl:param name="Chain"/>
    <xsl:choose>      
      <xsl:when test="not($Chain)">
        <xsl:copy>
          <xsl:apply-templates select="@* | node()" />
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="@* [name()='Chain'] | node()" >
          <xsl:with-param name="Chain" select="1"/>
          <xsl:with-param name="ChainSpline" select="1"/>
        </xsl:apply-templates>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- Template for l:Start-->
  <xsl:template match="l:Start">
    <xsl:param name="Chain"/>
    <xsl:choose>      
      <xsl:when test="not($Chain)">
              <xsl:copy>              
                <xsl:apply-templates select="@* | node()" />
              </xsl:copy>
      </xsl:when>
    <xsl:otherwise>
        <xsl:value-of select="concat(@pntRef, ' ')"/>
      </xsl:otherwise>
    </xsl:choose>    
  </xsl:template>
  
    <!-- Template for l:Center-->
  <xsl:template match="l:Center">
    <xsl:param name="Chain"/>          
      <xsl:if test="not($Chain)">
              <xsl:copy>              
                <xsl:apply-templates select="@* | node()" />
              </xsl:copy>
      </xsl:if>    
  </xsl:template>
  
  <!-- Template for l:End-->
  <xsl:template match="l:End">
    <xsl:param name="Chain"/>
    <xsl:param name="Last"/>
    <xsl:choose>      
      <xsl:when test="not($Chain)">
              <xsl:copy>              
                <xsl:apply-templates select="@* | node()" />
              </xsl:copy>
      </xsl:when>
    <xsl:otherwise>
        <xsl:if test="$Last">
        <xsl:value-of select="@pntRef"/>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>    
  </xsl:template>  
  
  <!-- Template for h:Start-->
  <xsl:template match="h:Start">
    <xsl:param name="Chain"/>
    <xsl:choose>      
      <xsl:when test="not($Chain)">
              <xsl:copy>              
                <xsl:apply-templates select="@* | node()" />
              </xsl:copy>
      </xsl:when>
    <xsl:otherwise>
        <xsl:value-of select="concat(@pntRef, ' ')"/>
      </xsl:otherwise>
    </xsl:choose>    
  </xsl:template>
  
  <!-- Template for h:Mid-->
  <xsl:template match="h:Mid">
    <xsl:param name="Chain"/>
    <xsl:choose>      
      <xsl:when test="not($Chain)">
              <xsl:copy>              
                <xsl:apply-templates select="@* | node()" />
              </xsl:copy>
      </xsl:when>
    <xsl:otherwise>
        <xsl:value-of select="concat(@pntRef, ' ')"/>
      </xsl:otherwise>
    </xsl:choose>    
  </xsl:template>
    
  <!-- Template for h:End-->
  <xsl:template match="h:End">
    <xsl:param name="Chain"/>
    <xsl:param name="Last"/>
    <xsl:choose>
      <xsl:when test="not($Chain)">
        <xsl:copy>              
          <xsl:apply-templates select="@* | node()" />
        </xsl:copy>
      </xsl:when>
      <xsl:otherwise>
        <xsl:if test="$Last">
          <xsl:value-of select="@pntRef"/>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>    
  </xsl:template>
  
<!-- Template for any Chain -->
  <xsl:template match="h:Chain | l:Chain">
    <xsl:param name="pointName"/>
    <xsl:param name="Chain"/>
    <xsl:param name="ChainSpline"/>
    <xsl:param name="inSearch"/>
    <xsl:choose>
      <xsl:when test="not($Chain)">
      <!-- Tokenize the text of the Chain searching the point name--> 
        <xsl:choose>
		  <xsl:when test="$inSearch">
            <xsl:apply-templates select="text()">
              <xsl:with-param name="toFind" select="$pointName"/>
            </xsl:apply-templates>
          </xsl:when>		
		  <xsl:otherwise>
            <xsl:copy>
              <xsl:value-of select="text()"/>
            </xsl:copy>
		  </xsl:otherwise>
		</xsl:choose>
      </xsl:when>
    <xsl:otherwise>      
      <xsl:choose>
        <xsl:when test="not($ChainSpline)">
          <!--best fit arc do not remove last point-->
          <xsl:value-of select="concat(text(), ' ')"/>
        </xsl:when>
        <xsl:otherwise>
          <!--spline - remove last point of chain-->
          <xsl:call-template name="substring-before-last"> 
            <xsl:with-param name="chaintext" select="text()"/> 
            <xsl:with-param name="delimiter" select="' '"/> 
          </xsl:call-template> 
          <xsl:value-of select="' '"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!--template to determine Substring before last occurence of a specific delemiter-->
  <xsl:template name="substring-before-last">     
    <xsl:param name="chaintext"/> 
    <xsl:param name="delimiter"/> 
    <xsl:choose> 
      <xsl:when test="contains($chaintext, $delimiter)"> 
        <!-- get all in front of the first delimiter --> 
        <xsl:value-of select="substring-before($chaintext,$delimiter)"/> 
        <xsl:choose> 
          <xsl:when test="contains(substring-after($chaintext,$delimiter),$delimiter)"> 
            <xsl:value-of select="$delimiter"/> 
          </xsl:when> 
        </xsl:choose> 
        <xsl:call-template name="substring-before-last"> 
        <!-- store anything left in another variable --> 
          <xsl:with-param name="chaintext" select="substring-after($chaintext,$delimiter)"/> 
          <xsl:with-param name="delimiter" select="$delimiter"/> 
        </xsl:call-template> 
      </xsl:when> 
    </xsl:choose> 
  </xsl:template> 
  
  <!-- This template tokenize the Chain text and set isFound when a CgPoint exist with the token -->
  <xsl:template match="h:Chain/text() | l:Chain/text()" name="tokenize">
    <xsl:param name="toFind"/>
    <xsl:param name="text" select="."/>
    <xsl:param name="separator" select="' '"/>	
      <xsl:choose>
        <xsl:when test="not(contains($text, $separator))">
          <xsl:if test="$toFind = $text and key('KeyCgPoint', $toFind)">isFound</xsl:if>
        </xsl:when>
        <xsl:otherwise>
          <xsl:variable name="token" select="normalize-space(substring-before($text, $separator))"/>
          <xsl:choose>
            <xsl:when test="$token = $toFind and key('KeyCgPoint', $token)">isFound</xsl:when>
            <xsl:otherwise>
              <xsl:call-template name="tokenize">
                <xsl:with-param name="text" select="substring-after($text, $separator)"/>
                <xsl:with-param name="toFind" select="$toFind"/>
              </xsl:call-template>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:otherwise>
      </xsl:choose>
  </xsl:template>
  
  <!-- Match free and thematical codes with attribute 'code'.
       (attribute 'code' do not exist in the elements of the Codelist -->
  <xsl:template match="h:FreeCode[@code] | h:PointCode[@code] | h:LineCode[@code]">
    <xsl:param name="Code"/>
    <xsl:choose>
	  <xsl:when test="not($Code)">
	    <xsl:copy>              
          <xsl:apply-templates select="@* | node()" />
        </xsl:copy>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:variable name="CodeName" select="@code"/>
	    <xsl:variable name="CodeInfo" select="@codeInformation"/>
	    <xsl:if test="$CodeInfo">
	      <xsl:value-of select="concat(@code, ' ', @codeInformation)"/>
	    </xsl:if>
	    <xsl:if test="not($CodeInfo)">
	      <xsl:value-of select="@code"/>
	    </xsl:if>
        <xsl:apply-templates select="h:Attribute">
          <xsl:with-param name="Code" select="1"/>
        </xsl:apply-templates>
        <xsl:apply-templates select="../h:Annotations">
          <xsl:with-param name="Code" select="1"/>
        </xsl:apply-templates>
	  </xsl:otherwise>
	</xsl:choose>    
  </xsl:template>

  <!-- Template for any Attribute -->  
  <xsl:template match="h:Attribute">
    <xsl:param name="Code"/>
    <xsl:choose>
	  <xsl:when test="not($Code)">
	    <xsl:copy>              
          <xsl:apply-templates select="@* | node()" />
        </xsl:copy>
	  </xsl:when>
	  <xsl:otherwise>
        <xsl:value-of select="concat(' ', @name, ' ', @value)"/>
	  </xsl:otherwise>
	</xsl:choose> 
  </xsl:template>
  
   <!-- Template for any Annotation -->  
  <xsl:template match="h:Annotations">
    <xsl:param name="Code"/>
    <xsl:choose>
	  <xsl:when test="not($Code)">
	    <xsl:copy>              
          <xsl:apply-templates select="@* | node()" />
        </xsl:copy>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:variable name="Annotation1" select="@annotation1"/>
	    <xsl:if test="$Annotation1">
	      <xsl:value-of select="concat(' ', @annotation1)"/>
	    </xsl:if> 
	    <xsl:variable name="Annotation2" select="@annotation2"/>
	    <xsl:if test="$Annotation2">
	      <xsl:value-of select="concat(' ', @annotation2)"/>
	    </xsl:if> 
	    <xsl:variable name="Annotation3" select="@annotation3"/>
	    <xsl:if test="$Annotation3">
	      <xsl:value-of select="concat(' ', @annotation3)"/>
	    </xsl:if> 
	    <xsl:variable name="Annotation4" select="@annotation4"/>
	    <xsl:if test="$Annotation4">
	      <xsl:value-of select="concat(' ', @annotation4)"/>
	    </xsl:if>        
	  </xsl:otherwise>
	</xsl:choose> 
  </xsl:template>

</xsl:stylesheet>